"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cling = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const DocumentOperationDescription_1 = require("./DocumentOperationDescription");
const EndCustomerOperationDescription_1 = require("./EndCustomerOperationDescription");
const GenericFunctions_1 = require("./GenericFunctions");
const ResourceDescription_1 = require("./ResourceDescription");
class Cling {
    constructor() {
        this.description = {
            displayName: 'Cling Node',
            name: 'cling',
            group: ['transform'],
            version: 1,
            subtitle: '={{ $parameter["resource"] }}',
            icon: 'file:cling.svg',
            description: 'Cling Node',
            defaults: {
                name: 'Cling',
            },
            credentials: [
                {
                    name: 'clingCredentialsApi',
                    required: true,
                },
            ],
            inputs: ['main'],
            outputs: ['main'],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    noDataExpression: true,
                    options: [
                        {
                            name: 'End Customer',
                            value: 'endCustomer',
                        },
                        {
                            name: 'Document',
                            value: 'document',
                        },
                    ],
                    default: 'document',
                },
                ...ResourceDescription_1.resourceDescription,
                ...EndCustomerOperationDescription_1.endCustomerOperationDescription,
                ...DocumentOperationDescription_1.documentOperationDescription,
            ],
        };
        this.methods = {
            loadOptions: {
                async getTemplates() {
                    const apiToken = await GenericFunctions_1.clingGetApiToken.call(this);
                    const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'get', 'template');
                    return (0, GenericFunctions_1.templateToOptions)(data.items);
                },
                async getEndCustomers() {
                    const apiToken = await GenericFunctions_1.clingGetApiToken.call(this);
                    const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'get', 'endCustomer');
                    return (0, GenericFunctions_1.toOptions)(data);
                },
                async getFields() {
                    const apiToken = await GenericFunctions_1.clingGetApiToken.call(this);
                    const templateId = this.getNodeParameter('templateId', '');
                    let fields;
                    if (templateId === '') {
                        const documentId = this.getNodeParameter('documentId', '');
                        if (documentId === '') {
                            throw new n8n_workflow_1.NodeOperationError(this.getNode(), `Cannot find fields.`);
                        }
                        else {
                            const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'get', `document/${documentId}`);
                            fields = data.template.validationSchema.properties.data.properties.fields.default;
                        }
                    }
                    else {
                        const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'get', `template/${templateId}`);
                        fields = data.validationSchema.properties.data.properties.fields.default;
                    }
                    const keys = Object.keys(fields);
                    const fieldOptions = [];
                    for (const key of keys) {
                        fieldOptions.push({
                            id: key,
                            name: fields[key].label,
                        });
                    }
                    return (0, GenericFunctions_1.fieldsToOptions)(fieldOptions);
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const apiToken = await GenericFunctions_1.clingGetApiToken.call(this);
        const returnItems = [];
        const resource = this.getNodeParameter('resource', 0, '');
        for (let itemIndex = 0; itemIndex < items.length; itemIndex++) {
            try {
                if (resource === "endCustomer") {
                    const id = this.getNodeParameter('endCustomerId', itemIndex, '');
                    const operation = this.getNodeParameter('operationEndCustomer', 0, '');
                    if ((operation !== "get" && id === "") && (operation !== "create" && id === "")) {
                        throw new n8n_workflow_1.NodeOperationError(this.getNode(), `Please enter the Id of the resource you want to perform the operation on`, {
                            itemIndex,
                        });
                    }
                    if (operation === "get") {
                        if (id === "") {
                            const filters = this.getNodeParameter('optionsEndCustomerGet', itemIndex, {});
                            const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'get', resource, {}, filters);
                            returnItems.push(...this.helpers.returnJsonArray(data));
                        }
                        else {
                            const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'get', `${resource}/${id}`, {}, {});
                            returnItems.push(...this.helpers.returnJsonArray(data));
                        }
                    }
                    if (operation === "create" || operation === "update") {
                        const bodyType = this.getNodeParameter('endCustomerBodyType', itemIndex, '');
                        let body = {};
                        if (bodyType === "perField") {
                            body = this.getNodeParameter('optionsEndCustomerPost', itemIndex, {});
                        }
                        else {
                            const tempBody = this.getNodeParameter('endCustomerPostBody', itemIndex, '');
                            try {
                                body = JSON.parse(tempBody);
                            }
                            catch {
                                throw new n8n_workflow_1.NodeOperationError(this.getNode(), `Cannot parse body.`, {
                                    itemIndex,
                                });
                            }
                        }
                        let method = 'post';
                        if (operation === 'update') {
                            method = 'put';
                        }
                        const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, method, `${resource}`, body, {});
                        returnItems.push(...this.helpers.returnJsonArray(data));
                    }
                    if (operation === "delete") {
                        const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'delete', `${resource}/${id}`, {}, {});
                        returnItems.push(...this.helpers.returnJsonArray(data));
                    }
                    if (operation === "restore") {
                        const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'patch', `${resource}/${id}`, {}, {});
                        returnItems.push(...this.helpers.returnJsonArray(data));
                    }
                }
                if (resource === "document") {
                    const id = this.getNodeParameter('documentId', itemIndex, '');
                    const operation = this.getNodeParameter('operationDocument', 0, '');
                    if ((operation !== "get" && id === "") && (operation !== "create" && id === "")) {
                        throw new n8n_workflow_1.NodeOperationError(this.getNode(), `Please enter the Id of the resource you want to perform the operation on`, {
                            itemIndex,
                        });
                    }
                    if (operation === "get") {
                        if (id === "") {
                            const filters = this.getNodeParameter('optionsdocumentGet', itemIndex, {});
                            const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'get', resource, {}, filters);
                            returnItems.push(...this.helpers.returnJsonArray(data.items));
                        }
                        else {
                            const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'get', `${resource}/${id}`, {}, {});
                            returnItems.push(...this.helpers.returnJsonArray(data));
                        }
                    }
                    if (operation === "create" || operation === "update") {
                        const bodyType = this.getNodeParameter('documentBodyType', itemIndex, '');
                        let fields = {};
                        if (bodyType === "perField") {
                            if (operation === "create") {
                                const tempFields = this.getNodeParameter('fields.field', itemIndex, []);
                                for (const field of tempFields) {
                                    fields[field.key] = { "value": field.value };
                                }
                            }
                            else {
                                const tempFields = this.getNodeParameter('updateFields.field', itemIndex, []);
                                for (const field of tempFields) {
                                    fields[`data.fields.${field.key}.value`] = field.value;
                                }
                            }
                        }
                        else {
                            const tempBody = this.getNodeParameter('documentPostBody', itemIndex, '');
                            try {
                                fields = JSON.parse(tempBody);
                            }
                            catch {
                                throw new n8n_workflow_1.NodeOperationError(this.getNode(), `Cannot parse body.`, {
                                    itemIndex,
                                });
                            }
                        }
                        let method = 'post';
                        let requestBody = {};
                        const qs = {};
                        let url = resource;
                        if (operation === 'update') {
                            method = 'put';
                            requestBody = fields;
                            const documentName = this.getNodeParameter('documentName', itemIndex, '');
                            if (documentName !== "") {
                                requestBody['data.name'] = documentName;
                            }
                            url = url + '/' + id;
                            qs.type = 'paths';
                        }
                        else {
                            const templateId = this.getNodeParameter('templateId', itemIndex, '');
                            const endCustomerId = this.getNodeParameter('endCustomerId', itemIndex, '');
                            const documentName = this.getNodeParameter('documentName', itemIndex, '');
                            requestBody.templateId = templateId;
                            requestBody.endCustomerId = endCustomerId;
                            requestBody.data = {
                                name: documentName,
                                fields
                            };
                        }
                        const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, method, `${url}`, requestBody, qs);
                        returnItems.push(...this.helpers.returnJsonArray(data));
                    }
                    if (operation === "delete") {
                        const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'delete', `${resource}/${id}`, {}, {});
                        returnItems.push(...this.helpers.returnJsonArray(data));
                    }
                    if (operation === "restore") {
                        const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'patch', `${resource}/${id}`, {}, {});
                        returnItems.push(...this.helpers.returnJsonArray(data));
                    }
                    if (operation === "send") {
                        const data = await GenericFunctions_1.clingApiRequest.call(this, apiToken, 'post', `${resource}/${id}/send`, {}, {});
                        returnItems.push(...this.helpers.returnJsonArray(data));
                    }
                }
            }
            catch (error) {
                if (this.continueOnFail()) {
                    items.push({ json: this.getInputData(itemIndex)[0].json, error, pairedItem: itemIndex });
                }
                else {
                    if (error.context) {
                        error.context.itemIndex = itemIndex;
                        throw error;
                    }
                    throw new n8n_workflow_1.NodeOperationError(this.getNode(), error, {
                        itemIndex,
                    });
                }
            }
        }
        return this.prepareOutputData(returnItems);
    }
}
exports.Cling = Cling;
//# sourceMappingURL=Cling.node.js.map